@@ -23,7 +23,7 @@ def test_sequence_matcher():
 def test_sequence_matcher_m2m():
     assert dpss.sequence_matcher_m2m(
         [1980, 2980, 3500, 4000, 1050],
-        [1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 10, 5, 10) == \
+        [1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 10) == \
         [[([1050], [1050]), ([1980], [30, 1950]), ([2980], [80, 2900]),
           ([3500], [200, 3300]), ([4000], [20, 3980])],
          [([1050, 1980, 2980], [80, 1950, 3980]), ([3500], [200, 3300]),
